function [positions, clockError] = LeastSquares(pData, pSoundsourcePositions, pCombinationDetails, pStartEndPosition)
%LEASTSQUARES A sample function to write your own tracking algorithms
%
%   Parameters:
%       pData (struct):
%           Data structured by soundsource. The toa is already corrected
%           (e.g. doppler correction etc.)
%       pSoundsourcePositions (struct):
%           The soundsource positions [deg].
%       pCombinationDetails (table):
%           Contains all information of the current soundsource
%           combination.
%       pStartEndPosition (double, logical):
%           Set to false if it has not been passed by artoa. Contains the
%           desired start and end position as a matrix of row vectors.
%
%
%   Returns:
%       positions (double):
%           The calculated list of segment positions as a X x 2 column vector.
%       clockError (double):
%           The estimated clock error.

%% Get reference position and all soundsource names that are involved
referencePosition = cellfun(@str2double, strsplit(pCombinationDetails.referencePosition{1}));
soundsourceNames = strsplit(pCombinationDetails.soundsources{1});

%% Prepare data for calculations
% get a date vector (does not matter which one)
dates = cell2mat(pData.(soundsourceNames{1}).date);
distances = [];
soundsourcePositions = [];
% prepare distances to, and positions of, all soundsources
for oSoundsource = 1:length(soundsourceNames)
    distances = [ ...
        distances, ...
        cell2mat(pData.(soundsourceNames{oSoundsource}).distance); ...
        ];
    soundsourcePositions = [ ...
        soundsourcePositions; ...
        pSoundsourcePositions.(soundsourceNames{oSoundsource}) ...
        ];
end

% initialize return variables
positions = referencePosition;
clockError = [];

% Calculate the actual segment positions
for oDates = 1:length(dates)
    positions = [positions; ...
        rad2deg( ...
            artoa.vendor.ls_converge( ...
                positions(end, :), ...
                length(soundsourceNames), ...
                soundsourcePositions, ...
                distances(oDates, :), ...
                pData.float.soundspeed/1000 ...
            ) ...
        ) ...
    ];
    clockError = [clockError; NaN(1, 3)];
    disp([' *** Date: ' num2str(dates(oDates)) ' converged at position: ' num2str(positions(end, 1)) ', ' num2str(positions(end, 2))]);
end

positions = positions(2:end, :);

end